==> lyskom-1.8.0.tar.gz: lyskom-1.8.0/src/server/server-config.c <==
/*
 * $Id: server-config.c,v 0.37 1995/11/02 21:49:02 ceder Exp $
 * Copyright (C) 1991, 1992, 1993, 1994, 1995  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 *  server-config.c
 *
 *  This is in a .c file to make it possible to change a value without having
 *  to recompile the entire server (or, in fact, anything!)
 *
 *  Compile with -DDEFAULT_PREFIX='"/usr/lyskom"' or something similar.
 */

static char *rcsid = "$Id: server-config.c,v 0.37 1995/11/02 21:49:02 ceder Exp $";
#include "rcs.h"
USE(rcsid);

#include <limits.h>
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <stdarg.h>
#include <assert.h>

#include "server/smalloc.h"
#include "config.h"
#include "server-config.h"
#include "misc-types.h"
#include "s-string.h"
#include "kom-types.h"
#include "param.h"
#include "conf-file.h"
#include "admin.h"
#include "log.h"
#include "manipulate.h"
#include "lyskomd.h"


static Success log_param(const char *val, const struct parameter *par);
static Success jubel(const char *val, const struct parameter *par);
static Success ident_param(const char *val, const struct parameter *par);

/* See lyskomd.8 for more info about the parameters.
   Please remember to update lyskomd.8 if you add more parameters!
   Try to keep this list and the list in lyskomd.8 in the same order. */
static const struct parameter parameters[] = {
    /* "Normal" configuration */
    {"Locale",
	 assign_string,  0, 1, NULL, &param.use_locale},
    {"Force ISO 8859-1",
         assign_bool,    0, 1, "no", &param.force_iso_8859_1},
    {"Prefix",
	 assign_string,  0, 1, DEFAULT_PREFIX, &param.dbase_dir},
    {"Send async",
  	 assign_bool,    0, 1, "1",  &param.send_async_messages},
    {"Client port",
	 assign_string,  1, 1, NULL, &param.ip_client_port},
    {"Mux port",   
	 assign_string,  1, 1, NULL, &param.ip_mux_port},
    {"Presentation of conferences",
	 assign_conf_no, 0, 1, "1", &kom_info.conf_pres_conf},
    {"Presentation of persons",
	 assign_conf_no, 0, 1, "2", &kom_info.pers_pres_conf},
    {"Motd-conference",
	 assign_conf_no, 0, 1, "3", &kom_info.motd_conf},
    {"News-conference", 
	 assign_conf_no, 0, 1, "4", &kom_info.kom_news_conf},
    {"Message of the day",
	 assign_text_no, 0, 1, "0", &kom_info.motd_of_lyskom},
    {"Garb",
	 assign_bool,    0, 1, "on", &param.garb_enable},
#ifndef NDEBUG
    {"Never save",
	 assign_bool,    0, 1, "no", &param.never_save},
#endif
#ifdef LOGACCESSES
    {"Log accesses", 
	 assign_string,  0, 1, NULL, &param.logaccess_file},
#endif
    /* The database files. */
    {"Data file",
	 assign_string,  0, 1, "db/lyskomd-data", &param.datafile_name},
    {"Backup file",
	 assign_string,  0, 1, "db/lyskomd-backup", &param.backupfile_name},
    {"Text file",
	 assign_string,  0, 1, "db/lyskomd-texts", &param.textfile_name},
    {"Text backup file",
	 assign_string,  0, 1, "db/lyskomd-texts-backup",
         &param.textbackupfile_name},
    /* Various log files */
    {"Log file",
	 assign_string, 0, 1, "etc/server-log", &param.logfile_name},
    {"Log statistics",
	 assign_string,  0, 1, "etc/lyskomd-log", &param.statistic_name},
    {"Pid file",
	 assign_string,  0, 1, "etc/pid", &param.pid_name},
    {"Memory usage file",
	 assign_string,  0, 1, "etc/memory-usage", &param.memuse_name},
    /* Where to dump core. */
    {"Core directory",
	 assign_string,  0, 1, "cores", &param.core_dir},
    /* Performance tuning parameters (milliseconds) */
    {"Idle timeout",
	 assign_int,     0, 1, "120000", &param.timeout},
    {"Garb timeout",
	 assign_int,     0, 1, "100", &param.garbtimeout},
    {"Sync timeout",
	 assign_int,     0, 1, "0", &param.synctimeout},
    /* Performance tuning parameters (minutes) */
    {"Garb interval",
	 assign_int,     0, 1, "1440", &param.garb_interval},
    {"Sync interval",
	 assign_int,     0, 1, "5", &param.sync_interval},
    {"Sync retry interval",
	 assign_int,     0, 1, "1", &param.sync_retry_interval},
    /* String limits */
    {"Max conference name length",
	 assign_int,     0, 1, "60", &param.conf_name_len},
    {"Max password length",
	 assign_int,     0, 1, "128", &param.pwd_len},
    {"Max what am I doing length",
	 assign_int,     0, 1, "60", &param.what_do_len},
    {"Max username length",
	 assign_int,     0, 1, "128", &param.username_len},
    {"Max text length",
	 assign_int,     0, 1, "131072", &param.text_len},
    {"Max broadcaset length",
	 assign_int,     0, 1, "1024", &param.broadcast_len},
    {"Max regexp length",
	 assign_int,     0, 1, "1024", &param.regexp_len},
    /* Text_stat limits */
    {"Max marks per person",
	 assign_int,     0, 1, "2048", &param.max_marks_person},
    {"Max marks per text",
	 assign_int,     0, 1, "1024", &param.max_marks_text},
    {"Max recipients per text",
	 assign_int,     0, 1, "512", &param.max_recipients},
    {"Max comments per text",
	 assign_int,     0, 1, "128", &param.max_comm},
    {"Max footnotes per text",
	 assign_int,     0, 1, "32", &param.max_foot},
    {"Max links per text",
	 assign_int,     0, 1, "512", &param.max_crea_misc},
    /* Other client-visible configuration */
    {"Max mark_as_read chunks",
	 assign_int,     0, 1, "128", &param.mark_as_read_chunk},
    {"Max super_conf loop",
	 assign_int,     0, 1, "17", &param.max_super_conf_loop},
    {"Default garb nice",
	 assign_int,     0, 1, "77", &param.default_nice},
    {"Max client transmit queue",
	 assign_int,     0, 1, "300", &param.maxqueuedsize},
    {"Max simultaneous client replies",
	 assign_int,     0, 1, "10", &param.maxdequeuelen},
    {"Open files",
         assign_int,     0, 1, "-1", &param.no_files},
    /* Security options */
    {"Anyone can create new persons",
	 assign_bool,    0, 1, "yes", &param.anyone_can_create_new_persons},
    {"Anyone can create new conferences",
	 assign_bool,    0, 1, "yes", &param.anyone_can_create_new_confs},
    {"Allow creation of persons before login",
	 assign_bool,    0, 1, "yes", &param.create_person_before_login},
    {"Default change name capability",
	 assign_bool,    0, 1, "on", &param.default_change_name},
    {"Log login",
	 assign_bool,    0, 1, "off", &param.log_login},
    /* Cache configuration */
    {"Cache conference limit",
	 assign_int,     0, 1, "20", &param.cache_conferences},
    {"Cache person limit",
	 assign_int,     0, 1, "20", &param.cache_persons},
    {"Cache text_stat limit",
	 assign_int,     0, 1, "20", &param.cache_text_stats},
    /* Echo the value to the log. */
    {"Echo",
	 log_param,      0, -1, NULL, NULL},
    /* Register a forbidden text number. */
    {"Jubel",
         jubel,          0, -1, NULL, NULL},
    {"Ident-authentication",
	ident_param,     0, 1, "try", &param.authentication_level},
    /* end marker */
    {NULL, NULL, 0, 0, NULL, NULL}};

/* Where to save things. These are used by lyskomd and dbck. */

/*
 * DEFAULT_DBASE_DIR can be overriden in the config file.  The default 
 * config file is found in DEFAULT_DBASE_DIR/CONFIG_FILE (before
 * DEFAULT_DBASE_DIR is overriden, of course).
 */

const char * DEFAULT_DBASE_DIR = DEFAULT_PREFIX;
const char *CONFIG_FILE = "etc/config";

/*
 * The following should always be true:
 * 0 <= SYNCTIMEOUT <= GARBTIMEOUT <= TIMEOUT
 * Times in milliseconds.
 */

/*
 * MAX_NO_OF_CONNECTIONS must be small enough.  See comment in config.h.
 */

#if defined(HAVE_GETDTABLESIZE) || defined(HAVE_SYSCONF)
int MAX_NO_OF_CONNECTIONS = 0;	/* Initialized by main(). */
#else
const int MAX_NO_OF_CONNECTIONS = (OPEN_MAX - PROTECTED_FDS);
#endif

/* What is whitespace? */
const unsigned char *WHITESPACE = (const unsigned char *)" \t\f\n\r";

static Success
log_param(const char *val, const struct parameter *par)
{
    if (val != NULL)
	log ("config: %s\n", val);
    return OK;
}

static Success
jubel(const char *val, const struct parameter *par)
{
    long a, b, c;
    int res;
    
    if (val == NULL)
        return OK;

    res = sscanf(val, "%ld %ld %ld", &a, &b, &c);
    switch (res)
    {
    case 3:
	register_jubel (a, b, c);
	break;
    case 2:
	register_jubel (a, 0, b);
	break;
    default:
	log ("%s expecting 2 or 3 integers as args\n", par->name);
	return  FAILURE;
    }
    return OK;
}

static Success
ident_param(const char *val, const struct parameter *par)
{
    if (val == NULL)
	restart_kom("ident_param(): val == NULL\n");
    if (!strcmp(val, "off")
	|| !strcmp(val, "never"))
    {
	*(int*)par->value = 0;
    }
    else if (!strcmp(val, "on")
	     || !strcmp(val, "try"))
    {
	*(int*)par->value = 1;
    }
    else if (!strcmp(val, "require")
	     || !strcmp(val, "required"))
    {
	*(int*)par->value = 2;
    }
    else
    {
	log ("%s expects \"never\", \"try\" or \"required\" as argument\n",
	     par->name);
	return FAILURE;
    }
    return OK;
}

static void
add_prefix(char **name)
{
    char *s;

    if (**name == '/')
	return;			/* Don't alter full paths. */

    s = smalloc(2 + strlen(param.dbase_dir) + strlen(*name));
    sprintf(s, "%s/%s", param.dbase_dir, *name);
    sfree(*name);
    *name = s;
}

const char *
param_name(void *value)
{
    int ix;

    for (ix = 0; parameters[ix].name != NULL; ix++)
	if (parameters[ix].value == value)
	    return parameters[ix].name;

    restart_kom("Internal error: non-existing config param in param_name.\n");
}

static Bool
check_abs_path(char **path)
{
    if (**path == '/')
	return FALSE;
	
    log("Parameter '%s' must be an absolute path when 'Prefix' is empty.\n",
	param_name(path));
    return TRUE;
}

void
read_configuration(const char *conf_file)
{
    Bool err = FALSE;

    read_config(conf_file, parameters);
    
    assert(param.dbase_dir != NULL);
    assert(param.datafile_name != NULL);
    assert(param.backupfile_name != NULL);
    assert(param.textfile_name != NULL);
    assert(param.textbackupfile_name != NULL);
    assert(param.statistic_name != NULL);
    assert(param.pid_name != NULL);
    assert(param.memuse_name != NULL);
    assert(param.logfile_name != NULL);
    assert(param.core_dir != NULL);

    if (strlen(param.dbase_dir) > 0) 
    {
	if (param.dbase_dir[0] != '/')
	{
	    log("The 'Prefix' parameter must be an absolute path.\n");
	    err = TRUE;
	}

	add_prefix(&param.datafile_name);
	add_prefix(&param.backupfile_name);
	add_prefix(&param.textfile_name);
	add_prefix(&param.textbackupfile_name);
	add_prefix(&param.statistic_name);
	add_prefix(&param.pid_name);
	add_prefix(&param.memuse_name);
	add_prefix(&param.logfile_name);
	add_prefix(&param.core_dir);
    }
    else
    {
	err |= check_abs_path(&param.datafile_name);
	err |= check_abs_path(&param.backupfile_name);
	err |= check_abs_path(&param.textfile_name);
	err |= check_abs_path(&param.textbackupfile_name);
	err |= check_abs_path(&param.statistic_name);
	err |= check_abs_path(&param.pid_name);
	err |= check_abs_path(&param.memuse_name);
	err |= check_abs_path(&param.logfile_name);
	err |= check_abs_path(&param.core_dir);
    }

    /* FIXME///+++ Check config parameters for sanity */

    if (err)
	restart_kom("Please fix the above configuration errors.\n");
}

==> lyskom-1.9.0.tar.gz: lyskom-1.9.0/src/server/server-config.c <==
/*
 * $Id: server-config.c,v 0.42 1996/08/04 01:30:17 ceder Exp $
 * Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 *  server-config.c
 *
 *  This is in a .c file to make it possible to change a value without having
 *  to recompile the entire server (or, in fact, anything!)
 *
 *  Compile with -DDEFAULT_PREFIX='"/usr/lyskom"' or something similar.
 */

static char *rcsid = "$Id: server-config.c,v 0.42 1996/08/04 01:30:17 ceder Exp $";
#include "rcs.h"
USE(rcsid);

#include <limits.h>
#include <stdio.h>
#include <sys/types.h>
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#include <time.h>
#include <stdarg.h>
#include <assert.h>

#include "server/smalloc.h"
#include "config.h"
#include "server-config.h"
#include "misc-types.h"
#include "s-string.h"
#include "kom-types.h"
#include "param.h"
#include "conf-file.h"
#include "admin.h"
#include "log.h"
#include "manipulate.h"
#include "lyskomd.h"


static Success log_param(const char *val, const struct parameter *par);
static Success jubel(const char *val, const struct parameter *par);
static Success ident_param(const char *val, const struct parameter *par);

/* See lyskomd.8 for more info about the parameters.
   Please remember to update lyskomd.8 if you add more parameters!
   Try to keep this list and the list in lyskomd.8 in the same order. */
static const struct parameter parameters[] = {
    /* "Normal" configuration */
    {"Locale",
	 assign_string,  0, 1, NULL, &param.use_locale},
    {"Force ISO 8859-1",
         assign_bool,    0, 1, "no", &param.force_iso_8859_1},
    {"Prefix",
	 assign_string,  0, 1, DEFAULT_PREFIX, &param.dbase_dir},
    {"Send async",
  	 assign_bool,    0, 1, "1",  &param.send_async_messages},
    {"Client port",
	 assign_string,  1, 1, NULL, &param.ip_client_port},
    {"Mux port",   
	 assign_string,  1, 1, NULL, &param.ip_mux_port},
    {"Presentation of conferences",
	 assign_conf_no, 0, 1, "1", &kom_info.conf_pres_conf},
    {"Presentation of persons",
	 assign_conf_no, 0, 1, "2", &kom_info.pers_pres_conf},
    {"Motd-conference",
	 assign_conf_no, 0, 1, "3", &kom_info.motd_conf},
    {"News-conference", 
	 assign_conf_no, 0, 1, "4", &kom_info.kom_news_conf},
    {"Message of the day",
	 assign_text_no, 0, 1, "0", &kom_info.motd_of_lyskom},
    {"Garb",
	 assign_bool,    0, 1, "on", &param.garb_enable},
#ifndef NDEBUG
    {"Never save",
	 assign_bool,    0, 1, "no", &param.never_save},
#endif
#ifdef LOGACCESSES
    {"Log accesses", 
	 assign_string,  0, 1, NULL, &param.logaccess_file},
#endif
    /* The database files. */
    {"Data file",
	 assign_string,  0, 1, "db/lyskomd-data", &param.datafile_name},
    {"Backup file",
	 assign_string,  0, 1, "db/lyskomd-backup", &param.backupfile_name},
    {"Text file",
	 assign_string,  0, 1, "db/lyskomd-texts", &param.textfile_name},
    {"Text backup file",
	 assign_string,  0, 1, "db/lyskomd-texts-backup",
         &param.textbackupfile_name},
    /* Various log files */
    {"Log file",
	 assign_string, 0, 1, "etc/server-log", &param.logfile_name},
    {"Log statistics",
	 assign_string,  0, 1, "etc/lyskomd-log", &param.statistic_name},
    {"Pid file",
	 assign_string,  0, 1, "etc/pid", &param.pid_name},
    {"Memory usage file",
	 assign_string,  0, 1, "etc/memory-usage", &param.memuse_name},
    /* Where to dump core. */
    {"Core directory",
	 assign_string,  0, 1, "cores", &param.core_dir},
    /* Performance tuning parameters (milliseconds) */
    {"Idle timeout",
	 assign_int,     0, 1, "120000", &param.timeout},
    {"Garb timeout",
	 assign_int,     0, 1, "100", &param.garbtimeout},
    {"Sync timeout",
	 assign_int,     0, 1, "0", &param.synctimeout},
    /* Performance tuning parameters (minutes) */
    {"Garb interval",
	 assign_int,     0, 1, "1440", &param.garb_interval},
    {"Permissive sync",
         assign_bool,    0, 1, "off", &param.permissive_sync},
    {"Sync interval",
	 assign_int,     0, 1, "5", &param.sync_interval},
    {"Sync retry interval",
	 assign_int,     0, 1, "1", &param.sync_retry_interval},
    /* String limits */
    {"Max conference name length",
	 assign_int,     0, 1, "60", &param.conf_name_len},
    {"Max password length",
	 assign_int,     0, 1, "128", &param.pwd_len},
    {"Max what am I doing length",
	 assign_int,     0, 1, "60", &param.what_do_len},
    {"Max username length",
	 assign_int,     0, 1, "128", &param.username_len},
    {"Max text length",
	 assign_int,     0, 1, "131072", &param.text_len},
    {"Max broadcaset length",
	 assign_int,     0, 1, "1024", &param.broadcast_len},
    {"Max regexp length",
	 assign_int,     0, 1, "1024", &param.regexp_len},
    /* Text_stat limits */
    {"Max marks per person",
	 assign_int,     0, 1, "2048", &param.max_marks_person},
    {"Max marks per text",
	 assign_int,     0, 1, "1024", &param.max_marks_text},
    {"Max recipients per text",
	 assign_int,     0, 1, "512", &param.max_recipients},
    {"Max comments per text",
	 assign_int,     0, 1, "128", &param.max_comm},
    {"Max footnotes per text",
	 assign_int,     0, 1, "32", &param.max_foot},
    {"Max links per text",
	 assign_int,     0, 1, "512", &param.max_crea_misc},
    /* Other client-visible configuration */
    {"Max mark_as_read chunks",
	 assign_int,     0, 1, "128", &param.mark_as_read_chunk},
    {"Max accept_async len",
	 assign_int,     0, 1, "128", &param.accept_async_len},
    {"Max super_conf loop",
	 assign_int,     0, 1, "17", &param.max_super_conf_loop},
    {"Default garb nice",
	 assign_int,     0, 1, "77", &param.default_nice},
    {"Max client transmit queue",
	 assign_int,     0, 1, "300", &param.maxqueuedsize},
    {"Max simultaneous client replies",
	 assign_int,     0, 1, "10", &param.maxdequeuelen},
    {"Open files",
         assign_int,     0, 1, "-1", &param.no_files},
    /* Security options */
    {"Anyone can create new persons",
	 assign_bool,    0, 1, "yes", &param.anyone_can_create_new_persons},
    {"Anyone can create new conferences",
	 assign_bool,    0, 1, "yes", &param.anyone_can_create_new_confs},
    {"Allow creation of persons before login",
	 assign_bool,    0, 1, "yes", &param.create_person_before_login},
    {"Default change name capability",
	 assign_bool,    0, 1, "on", &param.default_change_name},
    {"Log login",
	 assign_bool,    0, 1, "off", &param.log_login},
    /* Cache configuration */
    {"Cache conference limit",
	 assign_int,     0, 1, "20", &param.cache_conferences},
    {"Cache person limit",
	 assign_int,     0, 1, "20", &param.cache_persons},
    {"Cache text_stat limit",
	 assign_int,     0, 1, "20", &param.cache_text_stats},
    /* Echo the value to the log. */
    {"Echo",
	 log_param,      0, -1, NULL, NULL},
    /* Register a forbidden text number. */
    {"Jubel",
         jubel,          0, -1, NULL, NULL},
    {"Ident-authentication",
	ident_param,     0, 1, "try", &param.authentication_level},
    /* end marker */
    {NULL, NULL, 0, 0, NULL, NULL}};

/* Where to save things. These are used by lyskomd and dbck. */

/*
 * DEFAULT_DBASE_DIR can be overriden in the config file.  The default 
 * config file is found in DEFAULT_DBASE_DIR/CONFIG_FILE (before
 * DEFAULT_DBASE_DIR is overriden, of course).
 */

const char * DEFAULT_DBASE_DIR = DEFAULT_PREFIX;
const char *CONFIG_FILE = "etc/config";

/*
 * The following should always be true:
 * 0 <= SYNCTIMEOUT <= GARBTIMEOUT <= TIMEOUT
 * Times in milliseconds.
 */

/*
 * MAX_NO_OF_CONNECTIONS must be small enough.  See comment in config.h.
 */

#if defined(HAVE_GETDTABLESIZE) || defined(HAVE_SYSCONF)
int MAX_NO_OF_CONNECTIONS = 0;	/* Initialized by main(). */
#else
const int MAX_NO_OF_CONNECTIONS = (OPEN_MAX - PROTECTED_FDS);
#endif

/* What is whitespace? */
const unsigned char *WHITESPACE = (const unsigned char *)" \t\n\r";

static Success
log_param(const char *val, const struct parameter *par)
{
    if (val != NULL)
	log ("config: %s\n", val);
    return OK;
}

static Success
jubel(const char *val, const struct parameter *par)
{
    long a, b, c;
    int res;
    
    if (val == NULL)
        return OK;

    res = sscanf(val, "%ld %ld %ld", &a, &b, &c);
    switch (res)
    {
    case 3:
	register_jubel (a, b, c);
	break;
    case 2:
	register_jubel (a, 0, b);
	break;
    default:
	log ("%s expecting 2 or 3 integers as args\n", par->name);
	return  FAILURE;
    }
    return OK;
}

static Success
ident_param(const char *val, const struct parameter *par)
{
    if (val == NULL)
	restart_kom("ident_param(): val == NULL\n");
    if (!strcmp(val, "off")
	|| !strcmp(val, "never"))
    {
	*(int*)par->value = 0;
    }
    else if (!strcmp(val, "on")
	     || !strcmp(val, "try"))
    {
	*(int*)par->value = 1;
    }
    else if (!strcmp(val, "require")
	     || !strcmp(val, "required"))
    {
	*(int*)par->value = 2;
    }
    else
    {
	log ("%s expects \"never\", \"try\" or \"required\" as argument\n",
	     par->name);
	return FAILURE;
    }
    return OK;
}

static void
add_prefix(char **name)
{
    char *s;

    if (**name == '/')
	return;			/* Don't alter full paths. */

    s = smalloc(2 + strlen(param.dbase_dir) + strlen(*name));
    sprintf(s, "%s/%s", param.dbase_dir, *name);
    sfree(*name);
    *name = s;
}

const char *
param_name(void *value)
{
    int ix;

    for (ix = 0; parameters[ix].name != NULL; ix++)
	if (parameters[ix].value == value)
	    return parameters[ix].name;

    restart_kom("Internal error: non-existing config param in param_name.\n");
}

static Bool
check_abs_path(char **path)
{
    if (**path == '/')
	return FALSE;
	
    log("Parameter '%s' must be an absolute path when 'Prefix' is empty.\n",
	param_name(path));
    return TRUE;
}

void
read_configuration(const char *conf_file)
{
    Bool err = FALSE;

    read_config(conf_file, parameters);
    
    assert(param.dbase_dir != NULL);
    assert(param.datafile_name != NULL);
    assert(param.backupfile_name != NULL);
    assert(param.textfile_name != NULL);
    assert(param.textbackupfile_name != NULL);
    assert(param.statistic_name != NULL);
    assert(param.pid_name != NULL);
    assert(param.memuse_name != NULL);
    assert(param.logfile_name != NULL);
    assert(param.core_dir != NULL);

    if (strlen(param.dbase_dir) > 0) 
    {
	if (param.dbase_dir[0] != '/')
	{
	    log("The 'Prefix' parameter must be an absolute path.\n");
	    err = TRUE;
	}

	add_prefix(&param.datafile_name);
	add_prefix(&param.backupfile_name);
	add_prefix(&param.textfile_name);
	add_prefix(&param.textbackupfile_name);
	add_prefix(&param.statistic_name);
	add_prefix(&param.pid_name);
	add_prefix(&param.memuse_name);
	add_prefix(&param.logfile_name);
	add_prefix(&param.core_dir);
    }
    else
    {
	err |= check_abs_path(&param.datafile_name);
	err |= check_abs_path(&param.backupfile_name);
	err |= check_abs_path(&param.textfile_name);
	err |= check_abs_path(&param.textbackupfile_name);
	err |= check_abs_path(&param.statistic_name);
	err |= check_abs_path(&param.pid_name);
	err |= check_abs_path(&param.memuse_name);
	err |= check_abs_path(&param.logfile_name);
	err |= check_abs_path(&param.core_dir);
    }

    /* FIXME///+++ Check config parameters for sanity */

    if (err)
	restart_kom("Please fix the above configuration errors.\n");
}
